\sectiondef{DataDriven}{Simple data-driven computation}

%if False
\begin{code}
{-# OPTIONS -fglasgow-exts #-}
module DataDriven where

import Control.Arrow (second)
import Control.Monad (liftM2, join)
import Control.Applicative
import Control.Arrow (first)
import Data.Foldable (fold)
import Data.Monoid
import Data.Map hiding (fold)

import Data.IORef
import Graphics.UI.WX
import O

-- import Control.Instances () -- Monoid (IO a)
\end{code}
%endif

Imperative programs implement data-driven computation using two mechanisms: value extraction and change
notification.
Value extraction allows retrieval of a ``current value'' (e.g., via an input widget's access method).
Notification allows various states (e.g., an output widget) to be updated, making them consistent with newly changed values.
Our representation of data-driven computations encapsulates these two mechanisms, building them in tandem using a familiar set of combinators.

\subsection{Extractors}

Value extractors is represented simply as |IO| values.
\begin{code}
type Extractor = IO
\end{code}
For example, given a reference |r :: IORef Int|, define the extractor |rx = readIORef r|.
Or, given a slider widget |s|, define the extractor |sx = get selection s|.\footnote{The low-level GUI mechanisms are handled by wxHaskell~\cite{wxhaskell}.}

We can combine extractors applicatively.
For instance, the following function defines a ``sum'' of extractors, i.e., an extractor whose current value is the sum of the current values of given ones.
\begin{code}
plusX :: Num a => 
  Extractor a -> Extractor a -> Extractor a
plusX rx sx = do  r  <- rx
                  s  <- sx
                  return (r+s)
\end{code}
This code is quite tedious to write, so we would prefer to use the |liftM2| higher-order function, defined for monads:
\begin{spec}
plusX = liftM2 (+)
\end{spec}
Instead of this monad-based formulation, we use a more general formulation in terms of \out{the less familiar but more general setting of} ``applicative functors'' (AFs)~\cite{ApplicativeEffects}.
\footnote{The |Applicative| interface has just two operations: injection of a pure value and a form of function application.
\begin{spec}
class Functor f => Applicative f where
  pure   :: a -> f a
  (<*>)  :: f (a -> b) -> f a -> f b
\end{spec}
These primitives are used to define generalizations of the monadic |liftM|, |liftM2|, etc.
\begin{code}
liftA  f a      = pure f <*> a
liftA2 f a b    = liftA f a <*> b
liftA3 f a b c  = liftA2 f a b <*> c
...
\end{code}
}
The AF formulation of |plusX| looks much like the monadic formulation:
\begin{spec}
plusX = liftA2 (+)
\end{spec}
It's also easy to wrap up a regular value as an extractor.
Formulated monadically, we'd simply use |return|.
The more general AF formulation is ``|pure|''.
Thus, using AF methods, one can write arbitrarily rich applicative expressions to denote extractors.\notefoot{Consider adopting the AF sugar for this paper.}


\subsection{Notifiers}

For efficient data-driven computation, value extraction is not enough; we also need to construct change notifiers.
We will represent a notifier as the ability for clients to ``subscribe'' actions to be invoked whenever an event occurs.
\begin{code}
type Notifier = IO () -> IO ()
\end{code}
The following function is handy for creating nontrivial sources.
It makes a notifier, given a ``|setNotify|'' function that (destructively) assigns a single action to    be executed upon some event.
The subscribing actions are accumulated into a single, sequenced action held in a reference.\footnote{
Note that |join :: Monad m => m (m a) -> m a|, so |join| here turns an |IO (IO ())| into an |IO ()| that both reads the reference \emph{and} executes the contained value.
The last line returns an action that modifies the contents of the reference by sequencing its current action with a new one.
}
\begin{code}
mkNotifier :: Notifier -> IO Notifier
mkNotifier setNotify = 
  do  ref <- newIORef (return ())  -- subscribed actions
      setNotify (join (readIORef ref))
      return $ modifyIORef ref . (>>)
\end{code} %$
\out{For example, an ``active variable'' could be represented by one reference the current value and another for holding a single action to invoke.}
For example, imperative GUI toolkits come with a way to specify a ``callback'' action to invoke when a widget is modified.
Providing the widget and abstracting over the action gives a |setNotify| function suitable for passing to |mkNotifier|.
\begin{code}
cmdNotifier :: Commanding wid => wid -> IO Notifier
cmdNotifier wid =
  mkNotifier (\ act -> set wid [ on command := act ])
\end{code}

Given atomic notifiers (e.g., as constructed from a widget and |mkNotifier|), how do we build notifiers compositionally?
From notifiers |rn| and |sn|, we'd like to construct a composite notifier that reports a change whenever |rn| \emph{or} |sn| reports a change.
Longhand,
\begin{spec}
orN :: Notifier -> Notifier -> Notifier
orN rn sn = \ act -> rn act >> sn act
\end{spec}

We'll also want to make a notifier for never-occurring events, such as a pure (immutable) value changing.
Longhand,
\begin{code}
neverN :: Notifier
neverN = \ act -> return ()
\end{code}

Just as with extractors, we prefer to compose notifiers in terms of a more generic interface.
Instead of |Monad| or |Applicative|, we use |Monoid|.

Exploiting the |Monoid| instances for functions, |IO a|, and |()|, we have the following simple definitions.\footnote{The instances:
\begin{spec}
instance Monoid b => Monoid (a -> b) where
  mempty         = const mempty
  f `mappend` g  = \ x -> f x `mappend` g x
\end{spec}
\begin{code}
instance Monoid a => Monoid (IO a) where 
  mempty   = pure mempty
  mappend  = liftA2 mappend
\end{code}
\begin{spec}
instance Monoid () where
  mempty           = ()
  () `mappend` ()  = ()
\end{spec}
Note that the |IO| instance fits a more general pattern, in which |IO| may be replaced by \emph{any} AF. 
In particular, the function (|a -> b|) instance is also an example of this pattern, considering the
meanings of |pure| and |liftA2| for functions.
}
\begin{spec}
neverN  = mempty
orN     = mappend
\end{spec}
We now abandon the names ``|neverN|'' and ``|orN|'', and simply use ``|mempty|'' and ``|mappend|''.


\subsectiondef{SourceCombining}{Combining the pieces}

Our representation of data-driven computations pairs the representations given above for extractors and notifiers, into a ``source'' of values.
A single set of combinators works on both representations in tandem.
For reasons explained below, we will place the notifier first and apply a |newtype| constructor ``|O|''
to the pair.
For instance, a sum of two sources:
\begin{code}
addS :: Num a => Source a -> Source a -> Source a
addS (O (rn,rx)) (O (sn,sx)) =
  O (rn `mappend` sn, liftA2 (+) rx sx)
\end{code}

To make a \emph{source} from a value (unchanging) |v|,
\begin{code}
pureS :: a -> Source a
pureS a = O (mempty, pure a)
\end{code}

There is, again, a much more succinct formulation, made possible by casting |Source| as another AF.
\begin{spec}
addS   = liftA2 (+)
pureS  = pure
\end{spec}
The key to these simple definitions is to define |Source| as a type composition:
\begin{spec}
type Source = (,) Notifier `O` Extractor
\end{spec}
where type composition is defined as follows.
%include O.lhs

These composition properties are applicable because pairing with |Notifier| is an AF, which is the case exactly because |Notifier| is a monoid.
\out{(Then |(,) Notifier| acts like the \emph{writer} monad transformer.~\cite{jones95overloading}.)}
\begin{spec}
instance Functor ((,) u) where
  fmap f (u,x) = (u, f x)

instance Monoid u => Applicative ((,) u) where
  pure x             = (mempty, x)
  (u, f) <*> (v, x)  = (u `mappend` v, f x)
\end{spec}

By combining the instances for |g `O` f| with the instances for |(,) u| specialized to |Notifier|, it follows that, for sources,
\begin{spec}
  fmap f (O (rn,rx)) == O (rn, fmap f rx)

  pure a == O (mempty, pure a)

  O (nf,xf) <*> O (nx, xz) ==
    O (nf `mappend` nx) (xf <*> xz)
\end{spec}
Returning to the sum example above, the previous definitions of |addS| and |pureS| can now be derived.
\begin{spec}
    pureS a
==  pure a
==  O (pure (pure a))
==  O (mempty, pure a)
\end{spec}
and
\begin{spec}
    addS (O (rn,rx)) (O (sn,sx))
==  liftA2 (+) (O (rn,rx)) (O (sn,sx))
==  fmap (+) (O (rn,rx)) <*> O (sn,sx)
==  O (fmap (fmap (+)) (rn,rx)) <*> O (sn,sx)
==  O (rn, fmap (+) rx) <*> O (sn,sx)
==  O (liftA2 (<*>) (rn, fmap (+) rx) (sn,sx))
==  O (rn `mappend` sn, fmap (+) rx <*> sx)
==  O (rn `mappend` sn, liftA2 (+) rx sx)
\end{spec}


Beside |pure| and |(<*>)|, we can also construct sources explicitly.
For example, the following function presents a widget and input attribute as a source.
\begin{code}
attrSource  ::  Commanding wid =>
  wid -> Attr wid a ->  IO (Source a)
attrSource wid attr =
  do  nfy <- cmdNotifier wid
      return (O (nfy, get wid attr))
\end{code}

\subsection{Generalizing}

In fact, the |Functor| and |Applicative| instances for |Source| rely on very little about the choice of |IO| and |Notifier|, so they can be stated much more generally.
\begin{code}
type DataDriven nfr xtr = (,) nfr `O` xtr

type Source = DataDriven Notifier Extractor
\end{code}
With this refactoring, |DataDriven nfr xtr| is an AF for \emph{any} monoid |nfr| and applicative functor |xtr|.


\subsection{Running a data-driven computation}

We can ``run'' a source of actions by executing its current value whenever it changes.
\begin{code}
runDD :: Source (IO ()) -> IO ()
runDD (O (nfr,xtr)) = nfr act >> act
  where act = join xtr
\end{code}
Again, |join| here turns an |IO (IO ())| into |act :: IO ()|.
Executing |act| retrieves \emph{and} executes the current value of |xtr|.
The body of the definition subscribes |act| and executes it once up front, as initialization.

%if False
Consider this one.  If I keep it, then redefine |runDD|.
\begin{code}
joinDD :: Source (IO ()) -> Source ()
joinDD = inO (second join)
\end{code}
%endif


\subsectiondef{UNotifier}{Unique notification}

As defined above, notifiers can get invoked redundantly.
Consider |a+a|, where the source |a = O (na,xa) |.
The notifier would be |na `mappend` na|, which is equivalent to |\ act -> na act >> na act|.
That is, any subscribing action |act| would get invoked twice.

To eliminate redundant unification, represent notifiers as maps from unique tags to simple notifiers.
\begin{code}
type UNotifier  = Map Int Notifier
 
type USource    = DataDriven UNotifier Extractor
\end{code}
Since |Map k v| is a monoid whenever |Ord k| (e.g., |k == Int|), |USource| is an AF.
(The |mappend| operation for maps is a left-biased union.)

To convert from |Notifier| to |UNotifier|, make a singleton map with a given tag.
Conversely, to convert from |UNotifier| to |Notifier|, just forget the tags and combine the individual notifiers, which corresponds to the |fold| operation in the |Map k v| instance of |Foldable| type class (when |v| is a monoid).
Using these simple conversions, define conversions between |Source| and |USource| as follows.\footnote{The |first| function applies a given function to the first member of a pair: |first f (x,y) = (f x, y)|.  More generally, it applies to any arrow, not just to functions.}
\begin{code}
toUSource :: Int -> Source a -> USource a
toUSource tag = inO (first (singleton tag))

fromUSource :: USource a -> Source a
fromUSource = inO (first fold)
\end{code}

We'll need a way to generate generators of new tags:
\begin{code}
type NewTag = IO Int

newNewTag :: IO NewTag
newNewTag =
  do  symRef <- newIORef 0
      return (do  modifyIORef symRef (+1)
                  readIORef symRef)
\end{code}

\subsection{Revisiting extractors}

%format liftA_n = "liftA\!_n"
%format a_1 = "a\!_1"
%format a_n = " a\!_n"
We've used |Extractor == IO|, but extractors only \emph{read} state, they do not write it.
%As such, |liftA_n f a_1 ... a_n| is insensitive to order of extraction of the |a_i|.
As such, |fx <*> ax| is insensitive to order of extraction of |fx| vs |ax|.
Is there an alternative to |IO| that captures this property?nnnnn
