%% FlexibleLayout.lhs

%% Copyright (c) 2007 by Conal Elliott
%% Created: Tue May 15, 2007 16:20
%% Last Modified: Mon Jun 18, 2007 18:39 by Conal Elliott

\sectiondef{FlexibleLayout}{Flexible layout}

%if False
\begin{code}
{-# OPTIONS -fglasgow-exts #-}
module FlexibleLayout where

import Control.Applicative hiding (empty)
import Data.Monoid
import Control.Arrow (first,second)

import Graphics.UI.WX

import SimpleUI (above, leftOf, Unop, Win, hwidget)

import O
import DataDriven
\end{code}
%endif

So far, our generated GUIs are all laid out from top to bottom.
Next we add choice of layout with the ability to mix different layouts in a GUI.
The vital change is in the layout information generated for each GUI.
Rather than using a fixed |Layout| monoid (|empty| and |above|), GUIs will take the monoid specification from context.
\begin{code}
type UI' a = Win -> IO (CxLayout, Source a)

type UI = (->) Win `O` IO `O` (,) CxLayout `O` Source

type CxLayout = CxMonoid Layout

newtype CxMonoid a =
  CxMonoid { unCxMonoid :: MonoidDict a -> a }

type MonoidDict a = (a, a -> a -> a)

instance Monoid (CxMonoid a) where
  mempty = CxMonoid (\ (e,_) -> e)
  CxMonoid f `mappend` CxMonoid g  =
    CxMonoid (\ md@(_,op) -> f md `op` g md)
\end{code}
The definitions of |MonoidDict| and |CxMonoid|, as well as the |Monoid| instance for |CxMonoid|, are all mechanically derived from the |Monoid| type class.

As required for |UI| to be an applicative functor, |CxLayout| is a monoid.

%if False
\begin{code}
ui    :: UI'  a -> UI   a
unUI  :: UI   a -> UI'  a

ui    = O . O . O
unUI  = unO . unO . unO
\end{code}
%endif

Running a UI works as in \secref{SimpleUI}, except that the |MoinoidDict| |(empty,above)| is passed in to extract a layout.
%if False
\begin{code}
runUI :: UI (IO ()) -> IO ()
runUI = runNamedUI "Phooey GUI"
\end{code}

%% Maybe show this one.
\begin{code}
runNamedUI :: String -> UI (IO ()) -> IO ()
runNamedUI name ui = start $
  do  f          <- frame [ visible := False, text := name ]
      win        <- panel f []
      (cxl,src)  <- unUI ui win
      set win  [ layout := unCxMonoid cxl (empty,above) ]
      set f    [ layout := hwidget win, visible := True ]
      runDD src
\end{code} %$
%endif

%% Widgets
%if False
\begin{code}
type IWidget  a = a -> UI a
type OWidget  a = UI (a -> IO ())

type MkWidget wid a b =
  (Win -> [Prop wid] -> IO wid) -> Attr wid a -> b

stringEntry :: IWidget String
stringEntry = iwidget textEntry text

islider :: (Int,Int) -> IWidget Int
islider (lo,hi) =
  iwidget (\ win -> hslider win True lo hi) selection

stringDisplay :: OWidget String
stringDisplay = owidget textEntry text

showDisplay :: Show a => OWidget a
showDisplay = fmap (. show) stringDisplay

iwidget :: (Commanding wid, Widget wid) =>
   MkWidget wid a (IWidget a)

owidget :: Widget wid => MkWidget wid a (OWidget a)
owidget mkWid attr = ui $ \ win ->
  do  wid <- mkWid win [ ]
      return (widgetCXL wid, pure (\ a -> set wid [ attr := a ]))

\end{code} %$
%endif

The only change in widget creation (relative to \secref{widgets}) is that the new versions of
|iwidget| and |owidget|, use a new function |widgetCXL| that ignores an incoming |MonoidDict|.
\begin{code}
widgetCXL :: Widget w => w -> CxLayout
widgetCXL wid = CxMonoid (const (hwidget wid))
\end{code}
The |iwidget| and |owidget| functions use |widgetCXL| in place of |hwidget|.
%if False
For instance,
\begin{code}
iwidget mkWid attr initial = ui $ \ win ->
  do  wid  <- mkWid win [ attr := initial ]
      src  <- attrSource wid attr
      return (widgetCXL wid, src)
\end{code} %$
%endif

The pay-off in the new representation comes in definability of layout-altering functions.
For instance,
\begin{code}
leftToRight, topToBottom, flipLayout :: Unop (UI a)
leftToRight  = withCxMonoid (empty,leftOf)
topToBottom  = withCxMonoid (empty,above)
flipLayout   = compCxMonoid (second flip)
\end{code}
The |withCxMonoid| function overrides an inherited layout monoid, using the more general |compCxMonoid|.
\begin{code}
withCxMonoid :: MonoidDict Layout -> Unop (UI a)
withCxMonoid dict = compCxMonoid (const dict)

compCxMonoid :: Unop (MonoidDict Layout) -> Unop (UI a)
compCxMonoid f = onCxLayout' (. f)
\end{code} %$

The |onCxLayout'| function is defined on top of |onCxLayout| (analogous to |onLayout| from
\secref{titling}), adding and removing the |CxLayout| constructor.
%if False
\begin{code}
onCxLayout :: Unop CxLayout -> Unop (UI a)
onCxLayout f = ui . (fmap.fmap.first) f . unUI

onCxLayout' :: Unop (MonoidDict Layout -> Layout) -> Unop (UI a)
onCxLayout' f' = onCxLayout (CxMonoid . f' . unCxMonoid)
\end{code}
%endif

%if False
\begin{code}
title :: String -> Unop (UI a)
title str = onCxLayout' (boxed str .)
\end{code} %$
%endif

\subsection{Examples}

The examples in \secref{examples-simpl} all work as before.
%if False
\begin{code}
apples, bananas, fruit :: UI Int
apples   = title "apples"   $ islider (0,10) 3
bananas  = title "bananas"  $ islider (0,10) 7
fruit    = title "fruit"    $ liftA2 (+) apples bananas
 
total :: Show a => OWidget a
total = title "total" showDisplay

shopping :: UI (IO ())
shopping = title "Shopping List" $ fruit <**> total
\end{code}
%endif
In addition, \picrefdef{shopping-layout-variants}{Some layout variants} shows three variations, as defined below.
\begin{code}
shoppingFlip  = flipLayout shopping
shoppingLR    = leftToRight shopping
shoppingTLR   = leftToRight fruit <**> total
\end{code}
