%% SimpleUI.lhs

%% Copyright (c) 2007 by Conal Elliott
%% Created: Tue May 15, 2007 16:20
%% Last Modified: Mon Jul 30, 2007 13:16 by Conal Elliott

\section{GUIs, first version} \label{sec:SimpleUI}

%if False
\begin{code}
{-# OPTIONS -fglasgow-exts #-}
module SimpleUI where

import Control.Applicative hiding (empty)
import Data.Monoid
import Control.Arrow (first)

import Graphics.UI.WX

import O
import DataDriven
\end{code}
%endif

We represent GUIs as functions that take a container sub-window and produce a layout and a value
source.
\begin{code}
type Win  = Panel () -- widget container

type UI' a = Win -> IO (Layout, Source a)
\end{code}
From the first definition, we can see that |UI'| is a composition of four simpler components: sources, pairing with a layout, |IO|, and function from |Win|.
Writing this composition explicitly will make it easy to define UI operations.
\begin{code}
type UI = (->) Win `O` IO `O` (,) Layout `O` Source
\end{code}
These two types are isomorphic:
\begin{code}
ui    :: UI'  a -> UI   a
unUI  :: UI   a -> UI'  a

ui    = O    . O    . O
unUI  = unO  . unO  . unO
\end{code}

Recall from \secref{SourceCombining} that |UI| is an AF if the composed pieces are.
All four pieces are indeed AFs, assuming |Layout| is a monoid.
For now we'll provide a simple |Monoid| instance for |Layout|, stacking vertically:
\begin{code}
instance Monoid Layout where
  mempty   = empty
  mappend  = above

above, leftOf :: Layout -> Layout -> Layout
la `above`   lb = fill (column  0 [la,lb])
la `leftOf`  lb = fill (row     0 [la,lb])
\end{code}

%if False
To ``run'' a UI:
\begin{code}
runUI :: UI (IO ()) -> IO ()
runUI = runNamedUI "Phooey GUI"

runNamedUI :: String -> UI (IO ()) -> IO ()
runNamedUI name ui = start $
  do  f        <- frame [ visible := False, text := name ]
      win      <- panel f []
      (l,src)  <- unUI ui win
      set win  [ layout := l ]
      set f    [ layout := hwidget win, visible := True ]
      runDD src
\end{code} %$
%endif


\subsectiondef{widgets}{Widgets}

Input widget construction takes an initial value and makes a UI
\begin{code}
type IWidget  a = a -> UI a
\end{code}
For instance, a string entry widget:
\begin{code}
stringEntry :: IWidget String
\end{code}
Other parameters may be necessary as well, such as the value bounds for a slider.
\begin{code}
islider :: (Int,Int) -> IWidget Int
\end{code}

The definitions are easy, given an auxiliary function |iwidget|.
\begin{code}
stringEntry = iwidget textEntry text

islider (lo,hi) =
  iwidget (\ win -> hslider win True lo hi) selection
\end{code}

Beside the initial value, the function |iwidget| takes a widget-making function and a choice of
attribute.
Output widgets are created similarly, and the following type definition captures the commonality.
\begin{code}
type MkWidget wid a b =
  (Win -> [Prop wid] -> IO wid) -> Attr wid a -> b
\end{code}
Creation of input widgets is straightforward, using |attrSource|, from \secref{SourceCombining}.
\begin{code}
iwidget :: (Commanding wid, Widget wid) =>
   MkWidget wid a (IWidget a)

iwidget mkWid attr initial = ui $ \ win ->
  do  wid  <- mkWid win [ attr := initial ]
      src  <- attrSource wid attr
      return (hwidget wid, src)

hwidget :: Widget w => w -> Layout
hwidget = hfill . widget
\end{code} %$

While input widgets \emph{produce} values, output widgets \emph{consume} them.
\begin{code}
type OWidget a = UI (a -> IO ())

owidget :: Widget wid => MkWidget wid a (OWidget a)
owidget mkWid attr = ui $ \ win ->
  do  wid <- mkWid win [ ]
      return  (  hwidget wid
              ,  pure (\ a -> set wid [ attr := a ]))
\end{code} %$
The beauty of this definition of |OWidget| is that outputs (consumers) can simply be \emph{applied to} inputs (producers), using the central applicative functor operator, ``|<*>|''.

For instance, we can display a string or any showable value.
\begin{code}
stringDisplay :: OWidget String
stringDisplay = owidget textEntry text

showDisplay :: Show a => OWidget a
showDisplay = fmap (. show) stringDisplay
\end{code}

\subsectiondef{titling}{Titling}

%% %if False
%% \begin{code}
%% inUI' :: (UI' a -> UI' b) -> (UI a -> UI b)
%% -- inUI' f = ui . f . unUI
%% inUI' = inO.inO.inO
%% \end{code}
%% %endif

Adding a title to a GUI requires altering the layout produced.
The function |onLayout|, below, applies a given function to the layout part of a UI.
\begin{code}
type Unop a = a -> a

onLayout :: Unop Layout -> Unop (UI a)
onLayout f = ui . (fmap.fmap.first) f . unUI
\end{code}
The |fmap|s correspond to the the functors |(->) Win| and |IO|, and |first| to |(,) Layout|.\footnote{If we wanted to alter the value source, we would have used |second| or another |fmap| in place of |first|.}

Adding a title then is easy, using wxHaskell's function |boxed :: String -> Layout -> Layout|.
\begin{code}
title :: String -> Unop (UI a)
title str = onLayout (boxed str)
\end{code}


\subsectiondef{examples-simpl}{Examples}

As an example, \picrefdef{fruit-shopping-padded}{Simple GUI} is a simple shopping list GUI.
The total displayed at the bottom of the window always shows the sum of the values of the apples and bananas input sliders.
When a user changes the inputs, the output updates accordingly.

In the code below, note that |shopping| uses the reverse application operator (|<**>|).
This reversal causes the function to appear after (below) the argument.
\begin{code}
apples, bananas, fruit :: UI Int
apples   = title "apples"   $ islider (0,10) 3
bananas  = title "bananas"  $ islider (0,10) 7
fruit    = title "fruit"    $ liftA2 (+) apples bananas
 
total :: Show a => OWidget a
total = title "total" showDisplay

shopping :: UI (IO ())
shopping = title "Shopping List" $ fruit <**> total
\end{code}

%if False
\begin{code}
lam' :: UI a -> UI (b -> o) -> UI ((a->b) -> o)
lam' ia ob = ui $ \ win ->
  do  (la, sa)  <- unUI ia win
      (lb, sb)  <- unUI ob win
      return (   la `above` lb
             ,   hh <$> sa <*> sb )
 where
   hh a snkb = \ f -> snkb (f a)

-- simply and generalize.
lam :: Applicative f => f a -> f (b -> o) -> f ((a->b) -> o)
lam = liftA2 (\ a snkb -> \ f -> snkb (f a))
\end{code}
%endif
